/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace MobiServer.Devices.Phidget.Control
{
    public partial class Accel : Base
    {
        NumericUpDown[] numericAccels = null;
        NumericUpDown[] numericAccelTs = null;
        NumericUpDown[] numericAccelZs = null;
        Phidgets.Accelerometer phidgetAccel;

        public Accel(Phidgets.Phidget phidget) 
            : base(phidget) 
        {
            InitializeComponent();
            InitComponentArray();
            phidgetAccel = (Phidgets.Accelerometer)phidget;
            LoadSetting();
        }

       
        void InitComponentArray()
        {

            numericAccels = new System.Windows.Forms.NumericUpDown[3];
            numericAccels[0] = numericUpDownAccelX;
            numericAccels[1] = numericUpDownAccelY;
            numericAccels[2] = numericUpDownAccelZ;

            numericAccelTs = new System.Windows.Forms.NumericUpDown[3];
            numericAccelTs[0] = numericUpDownThresholdX;
            numericAccelTs[1] = numericUpDownThresholdY;
            numericAccelTs[2] = numericUpDownThresholdZ;

            numericAccelZs = new System.Windows.Forms.NumericUpDown[3];
            numericAccelZs[0] = numericUpDownZeroX;
            numericAccelZs[1] = numericUpDownZeroY;
            numericAccelZs[2] = numericUpDownZeroZ;

            return;
        }
     
        override public bool Start()
        {
            try
            {          
                if (!CheckDeviceAttached())
                {
                    return false;
                }

                PrintType();

                for (int i = 0; i < numericAccels.Length; i++)
                {
                    if (i < phidgetAccel.axes.Count)
                    {
                        numericAccels[i].Enabled = true;
                        numericAccelTs[i].Enabled = true;
                        numericAccelZs[i].Enabled = true;
                        phidgetAccel.axes[i].Sensitivity = (double)numericAccelTs[i].Value;
                    }
                    else
                    {
                        numericAccels[i].Enabled = false;
                        numericAccelTs[i].Enabled = false;
                        numericAccelZs[i].Enabled = false;
                    }
                }

                phidgetAccel.AccelerationChange +=
                    new Phidgets.Events.AccelerationChangeEventHandler(OnAccelerationChange);
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
                return false;
            }
            return true;
        }

        void OnAccelerationChange(object sender, Phidgets.Events.AccelerationChangeEventArgs args)
        {
        
            if (args.Index < phidgetAccel.axes.Count)
            {
                double data = args.Acceleration - (double)numericAccelZs[args.Index].Value;
                PrintDebug("{0}:{1:000.000}", args.Index, data);
          
                numericAccels[args.Index].Value = (decimal)data;
                Application.DoEvents();
                Data.Accel accel = new Data.Accel(SerialNumber, Data.States.In, args.Index, data);
                CreateServerWriteEvent(accel.ToArray());
            }
        }

        private void numericUpDownThresholdX_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetAccel != null && phidgetAccel.Attached)
            {
                phidgetAccel.axes[0].Sensitivity = (double)numericUpDownThresholdX.Value;
            }
        }

        private void numericUpDownThresholdY_ValueChanged(object sender, System.EventArgs e)
        {
            if (phidgetAccel != null && phidgetAccel.Attached)
            {
                phidgetAccel.axes[1].Sensitivity = (double)numericUpDownThresholdY.Value;
            }
        }

        private void numericUpDownThresholdZ_ValueChanged(object sender, EventArgs e)
        {
            if (phidgetAccel != null && phidgetAccel.Attached)
            {
                phidgetAccel.axes[2].Sensitivity = (double)numericUpDownThresholdZ.Value;
            }
        }

        override public void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckDeviceType(e.Args))
                {
                    Data.Accel a = Data.Accel.Parse(e.Args);
                    switch (a.State)
                    {
                        case Data.States.Set:
                            if (phidgetAccel.axes.Count > a.Port)
                            {
                                switch (a.SubType)
                                {
                                    case Data.Accel.SubTypes.Threshold:
                                        numericAccelTs[a.Port].Value = (decimal)a.Data;
                                        break;
                                    case Data.Accel.SubTypes.Zero:
                                        numericAccelZs[a.Port].Value = (decimal)a.Data;
                                        break;
                                }
                            }
                            break;
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }

        #region Save/Load
        
        override public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();

            try
            {
                fileStream = new StreamWriter(file, false);

                for (int i = 0; i < numericAccelTs.Length; i++)
                {
                    fileStream.Write("{0}\t", numericAccelTs[i].Value);
                }
                fileStream.WriteLine();

                for (int i = 0; i < numericAccelZs.Length; i++)
                {
                    fileStream.Write("{0}\t", numericAccelZs[i].Value);
                }
                fileStream.WriteLine();
    
            }
            catch (IOException ex)
            {
                PrintError(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Saving " + Path.GetFileName(file));
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();

            try
            {
                if (File.Exists(file))
                {

                    fileStream = new StreamReader(file);

                    string line = null;
                    string[] args = null;

                    line = fileStream.ReadLine();

                    args = line.Split("\t".ToCharArray());

                    for (int i = 0; i < numericAccelTs.Length; i++)
                    {
                        numericAccelTs[i].Value = decimal.Parse(args[i]);
                    }


                    line = fileStream.ReadLine();
                    args = line.Split("\t".ToCharArray());
                    for (int i = 0; i < numericAccelZs.Length; i++)
                    {
                        numericAccelZs[i].Value = decimal.Parse(args[i]);
                    }
                }

            }
            catch (IOException ex)
            {
                PrintMessage(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Loading " + Path.GetFileName(file));

            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        #endregion
        
    }
}

