/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace MobiServer.Devices.Phidget.Control
{

    /// <summary>
    /// Under Developing... Not worked yet.
    /// </summary>
    public partial class Encoder : Base
    {

        Phidgets.Encoder phidgetEncoder = null;
        public Encoder(Phidgets.Phidget phidget) 
            : base(phidget) 
        {
            InitializeComponent();
            phidgetEncoder = (Phidgets.Encoder)phidget;
            LoadSetting();
        }

        override public bool Start()
        {
            try
            {
                
                if (!CheckDeviceAttached())
                    return false;
             
                PrintType();                
                phidgetEncoder.PositionChange += new Phidgets.Events.EncoderPositionChangeEventHandler(OnPositionChange);
                phidgetEncoder.InputChange +=new Phidgets.Events.InputChangeEventHandler(OnInputChange);

            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
                return false;
            }
            return true;
        }

        void OnPositionChange(object sender, Phidgets.Events.EncoderPositionChangeEventArgs args)
        {
            PrintDebug("{0}:{1}:{2}", args.Index, args.PositionChange, args.Time);

            if (args.Index > 0)
                return;

            trackBarEncoder.Value += args.PositionChange;
 
        }

        void OnInputChange(object sender, Phidgets.Events.InputChangeEventArgs args)
        {
            PrintDebug("{0}:{1:000.000}", args.Index, args.Value);

            if (args.Index > 0)
                return;

            checkBoxInput.Checked = args.Value;
        }

        private void trackBarEncoder_ValueChanged(object sender, System.EventArgs e)
        {
            numericUpDownEncoder.Value = trackBarEncoder.Value;     
        }

        public override void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckDeviceType(e.Args))
                {

                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }

        #region Save/Load
       
        override public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();

            try
            {
                fileStream = new StreamWriter(file, false);

            }
            catch (IOException ex)
            {
                PrintError(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Saving " + Path.GetFileName(file));
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();

            try
            {
                if (File.Exists(file))
                {

                    fileStream = new StreamReader(file);

                    string line = null;
                    string[] args = null;
                    line = fileStream.ReadLine();
                    args = line.Split("\t".ToCharArray());
                }
            }
            catch (IOException ex)
            {
                PrintMessage(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Loading " + Path.GetFileName(file));

            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }


        #endregion
    }
}

