/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Text;

using System.Text.RegularExpressions;

namespace MobiServer.Devices.Phidget.Data
{
    public class InterfaceKit : Base
    {
        protected string subType = "";
        int port = 0;
        int data = 0;

        public const int PORT_MAX = 8;
        //public const int DATA_MAX = 1024;

        public class SubTypes
        {
            public const string Digital = "Digital";
            public const string Analog = "Analog";
            public const string Threshold = "Threshold";
        }

        static string GetSubType(string type)
        {
            Regex regex = new Regex(type, RegexOptions.IgnoreCase);

            if (regex.Match(SubTypes.Digital).Success)
                type = SubTypes.Digital;
            else if (regex.Match(SubTypes.Analog).Success)
                type = SubTypes.Analog;
            else if (regex.Match(SubTypes.Threshold).Success)
                type = SubTypes.Threshold;

            return type;
        }

        public InterfaceKit(int serialNumber, string state, string subType, int port, int data)
            : base(Types.InterfaceKit, serialNumber, state)
        {
            this.subType = GetSubType(subType);
            this.port = port;
            this.data = data;
        }

        public InterfaceKit(int serialNumber, string state, int port, int data)
            : base(Types.InterfaceKit, serialNumber, state)
        {
            switch (state)
            {
                case States.Out:
                    this.subType = SubTypes.Digital;
                    break;
                case States.Set:
                    this.subType = SubTypes.Threshold;
                    break;
                case States.In:
                    this.subType = SubTypes.Analog;
                    break;
            }
            this.port = port;
            this.data = data;
        }

        public InterfaceKit(int serialNumber, string state)
            : base(Types.InterfaceKit, serialNumber, state)
        {
        }

        public string[] ToArray()    
        {
            string [] args = null;
            string [] headers = GetHeader();
            switch (state)
            {
                case States.Attach:
                case States.Detach:
                    args = base.ToArray();
                    break;
                case States.In:
                    int length = headers.Length;
                    args = new string[length + 3];
                    headers.CopyTo(args, 0);
                    args[length + 0] = subType;
                    args[length + 1] = port.ToString();
                    args[length + 2] = data.ToString();

                    break;
            }
            return args;
        }

        public static InterfaceKit Parse(string [] args)
        {
            try
            {
                int serialNumber = ParseSerialNumber(args);
                string state = ParseState(args);
                string subType = args[4];
                int port = int.Parse(args[5]);
                int data = int.Parse(args[6]);

                InterfaceKit interfaceData = new InterfaceKit(serialNumber, state, subType, port, data);
                return interfaceData;
            }
            catch (Exception ex)
            {
                return null;
            }
        }

        public string SubType
        {
            get { return this.subType; }
        }

        public int Port
        {
            get { return port; }
        }

        public int Data
        {
            get { return data; }
        }
    }
}