﻿/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;

namespace MobiServer
{
    public partial class PhidgetForm : Form
    {
        #region Variables

        Hashtable phidgetControlHolder = null;
        Devices.Phidget.Control.Base targetPhidgetControl = null;
        int targetHeight = 0;

        //Thread managerThread = null;
        //const int MANAGER_INIT_TIME = 1000;

        PhidgetManager phidgetManager = null;
        bool initialized = false;

        MobiServer.MessageHandler messageHandler;

        public bool Initialized
        {
            get { return initialized; }
        }

        #endregion

      
		#region Initialize/Finalize

        public PhidgetForm()
        {
            InitializeComponent();
            System.Windows.Forms.Control.CheckForIllegalCrossThreadCalls = false;
            phidgetControlHolder = new Hashtable();
        }

        private void PhidgetForm_Load(object sender, EventArgs e)
        {
            try
            {
                messageHandler += new MobiServer.MessageHandler(messageControl.OnMessageEvent);
                serverControl.messageHandler += new MobiServer.MessageHandler(messageControl.OnMessageEvent);
                StartManager();
                initialized = true;
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show("PhidgetServerの初期化に失敗しました．プログラムを終了します．\r\n" +
                                                     "Phidgetドライバがインストールされているか確認してください．\r\n", "初期化エラー");
                this.Dispose();
            }
        }

        void StartManager()
        {
            //Thread.Sleep(MANAGER_INIT_TIME);
            phidgetManager = new PhidgetManager(this);
        }

       
        public void StartControl(Devices.Phidget.Control.Base phidgetControl)
        {
            phidgetControl.messageHandler += new MobiServer.MessageHandler(messageControl.OnMessageEvent);
            phidgetControl.serverWriteHandler += new MobiServer.WriteHandler(serverControl.OnServerWriteEvent);
            serverControl.AddReadHandler(new MobiServer.ReadHandler(phidgetControl.OnServerReadEvent));
            phidgetControl.Start();
            AddControl(phidgetControl);
         }

        internal void WriteAllSockets(string[] args)
        {
            serverControl.WriteAllSockets(args);
        }

        public void AddControl(Devices.Phidget.Control.Base control)
        {
            Hashtable phidgetControlLine = null;
            bool addCompleted = false;
            
            for (int i = 0; i < phidgetControlHolder.Count; i++)
            {
                phidgetControlLine = (Hashtable)phidgetControlHolder[i];
                for (int j = 0; j < phidgetControlLine.Count; j++)
                {
                    Devices.Phidget.Control.Base c = (Devices.Phidget.Control.Base)phidgetControlLine[j];
                   
                    if (c.DeviceName == control.DeviceName &&
                                 c.SerialNumber == control.SerialNumber)
                    {
                        int num = this.Controls.IndexOf(c);
                        c.SetTitle();
                        this.Controls[num].Enabled = true;
                        this.Refresh();
                        return;
                    }
                }
            }
            
            for (int i = 0; i < phidgetControlHolder.Count; i++)
            {
                int lineWidth = 0;
                int y = 0;
                int lineHeight = 0;

                phidgetControlLine = (Hashtable)phidgetControlHolder[i];
                for (int j = 0; j < phidgetControlLine.Count; j++)
                {
                    Devices.Phidget.Control.Base c = (Devices.Phidget.Control.Base)phidgetControlLine[j];

                    if (!addCompleted)
                    {
                        lineWidth += c.Width;
                        y = c.Location.Y;
                        if (lineHeight < c.Height)
                            lineHeight = c.Height;
                    }
                    else
                    {
                        c.Location = new Point(c.Location.X, c.Location.Y + targetHeight);
                        phidgetControlLine[j] = c;
                    }
                }
                if (addCompleted)
                {
                    phidgetControlHolder[i] = phidgetControlLine;
                }
                else
                {
                    if (lineWidth + control.Width < this.Width)
                    {
                        control.Location = new Point(lineWidth, y);
                        targetPhidgetControl = control;

                        if (lineHeight < control.Height)
                            targetHeight = control.Height - lineHeight;
                        else
                            targetHeight = 0;

                        this.Invoke(new EventHandler(AddControl));
                        phidgetControlLine.Add(phidgetControlLine.Count, control);
                        phidgetControlHolder[i] = phidgetControlLine;
                        addCompleted = true;
                        //return;
                    }
                }
            }

            if (!addCompleted)
            {
                control.Location = new Point(0, this.Height - 30);// - 60
                targetPhidgetControl = control;
                targetHeight = targetPhidgetControl.Height;
                this.Invoke(new EventHandler(AddControl));  
                phidgetControlLine = new Hashtable();
                phidgetControlLine.Add(phidgetControlLine.Count, control);
                phidgetControlHolder.Add(phidgetControlHolder.Count, phidgetControlLine);
            }
           
            return;
        }

        void AddControl(object sender, EventArgs e)
        {
            this.SuspendLayout();
            this.Controls.Add(targetPhidgetControl);
            if (CheckScreenHeight(this.Height + targetHeight))
                this.Height += targetHeight;
            else
                this.Height = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height;
            this.ResumeLayout(true);
        }

        bool CheckScreenHeight(int height)
        {
            if (height <= System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height)
            {
                return true;
            }
            else
            {
                return false;
            }
        }


        public void DisableControl(Devices.Phidget.Control.Base control)
        {
            for (int i = 0; i < phidgetControlHolder.Count; i++)
            {
                int y = 0;

                Hashtable phidgetControlLine = (Hashtable)phidgetControlHolder[i];

                for (int j = 0; j < phidgetControlLine.Count; j++)
                {
                    Devices.Phidget.Control.Base c = (Devices.Phidget.Control.Base)phidgetControlLine[j];

                    if (c.DeviceName == control.DeviceName &&
                                c.SerialNumber == control.SerialNumber)
                    {
                        c.Enabled = false;
                        c.SaveSetting();

                        int num = this.Controls.IndexOf(c);
                        this.Controls[num].Enabled = false;
                        this.Refresh();
                        return;
                    }
                }
            }
            return;
        }


        #endregion

		#region Save/Load

        void SaveSettingAll()
        {
            Hashtable phidgetControlLine = null;

            for (int i = 0; i < phidgetControlHolder.Count; i++)
            {
                phidgetControlLine = (Hashtable)phidgetControlHolder[i];

                for (int j = 0; j < phidgetControlLine.Count; j++)
                {
                    Devices.Phidget.Control.Base c = (Devices.Phidget.Control.Base)phidgetControlLine[j];
                    c.SaveSetting();
                }
            }

            messageControl.SaveSetting();
            serverControl.SaveSetting();
        }

		#endregion

        #region Message Events

        public void PrintError(string m, params object[] args)
        {
            MobiServer.MessageEventArgs e = new MobiServer.MessageEventArgs
                                                    (String.Format(m, args), MobiServer.States.Error);
            if (messageHandler != null)
                messageHandler(this, e);
        }

        //メッセージイベントを起こす．
        public void PrintMessage(string m, params object[] args)
        {
            MobiServer.MessageEventArgs e = new MobiServer.MessageEventArgs(String.Format(m, args));

            if (messageHandler != null)
                messageHandler(this, e);
        }

        public void PrintDebug(string m, params object[] args)
        {
            MobiServer.MessageEventArgs e = new MobiServer.MessageEventArgs
                                                    (String.Format(m, args), MobiServer.States.Debug);

            if (messageHandler != null)
                messageHandler(this, e);
        }

        #endregion

     

    }
}
